﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Grids;

namespace ColumnBindingProxy
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = new MainVM();
            
        }
    }

    public class MainVM
    {

        public MainVM()
        {

            Cities = new List<City>
            {
                new City { ID = 1, Name = "Sofia", },
                new City { ID = 2, Name = "Karlovo", },
                new City { ID = 3, Name = "Plovdiv", },
            };



            DataSource = new List<Data>
            {
                new Data { Name = "item 1", City = Cities[0], },
                new Data { Name = "item 2", City = Cities[1], },
                new Data { Name = "item 3", City = Cities[2], },
            };
        }

        public List<Data> DataSource { get; set; }

        public List<City> Cities { get; set; }

    }

    public class Data
    { 
        public string Name { get; set; }
        public City City { get; set; }
    }

    public class City
    {
        public int ID { get; set; }
        public string Name { get; set; }
    }


    #region Field Binding Proxy

    public class FieldBindingProxy : DependencyObject
    {
        #region Private Fields
        private Column _owner;
        private XamGrid _grid;
        #endregion

        #region Initializing Methods
        public void AttachOwner(Column field)
        {
            if (_owner != null)
            {
                DetachOwner(_owner);
            }

            _owner = field;
            Dispatcher.BeginInvoke(new Action(() =>
            {
                _grid = _owner.ColumnLayout.Grid;
                InitializeAndSetBinding();
            }));
        }

        public void DetachOwner(Column field)
        {
            if (_owner != field)
            {
                return;
            }
            BindingOperations.ClearBinding(_owner, typeof(Column).GetField(FieldPropertyPath + "Property").GetValue(null) as DependencyProperty);

            _owner = null;
            _grid = null;
        }
        #endregion

        #region Properties

        #region BindingSource
        public BindingSource BindingSource
        {
            get { return (BindingSource)GetValue(BindingSourceProperty); }
            set { SetValue(BindingSourceProperty, value); }
        }

        public static readonly DependencyProperty BindingSourceProperty =
            DependencyProperty.Register(
                "BindingSource",
                typeof(BindingSource),
                typeof(FieldBindingProxy),
                new PropertyMetadata(BindingSource.DataContext,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));
        #endregion

        #region PropertyPath

        public string PropertyPath
        {
            get { return (string)GetValue(PropertyPathProperty); }
            set { SetValue(PropertyPathProperty, value); }
        }

        public static readonly DependencyProperty PropertyPathProperty =
            DependencyProperty.Register(
                "PropertyPath",
                typeof(string),
                typeof(FieldBindingProxy),
                new PropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));


        #endregion

        #region Converter

        public IValueConverter Converter
        {
            get { return (IValueConverter)GetValue(ConverterProperty); }
            set { SetValue(ConverterProperty, value); }
        }

        public static readonly DependencyProperty ConverterProperty =
            DependencyProperty.Register(
                "Converter",
                typeof(IValueConverter),
                typeof(FieldBindingProxy),
                new PropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));


        #endregion

        #region ConverterParameter


        public object ConverterParameter
        {
            get { return (object)GetValue(ConverterParameterProperty); }
            set { SetValue(ConverterParameterProperty, value); }
        }

        public static readonly DependencyProperty ConverterParameterProperty =
            DependencyProperty.Register(
                "ConverterParameter",
                typeof(object),
                typeof(FieldBindingProxy),
                new UIPropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));


        #endregion

        #region FieldPropertyPath

        public string FieldPropertyPath
        {
            get { return (string)GetValue(FieldPropertyPathProperty); }
            set { SetValue(FieldPropertyPathProperty, value); }
        }

        public static readonly DependencyProperty FieldPropertyPathProperty =
            DependencyProperty.Register(
                "FieldPropertyPath",
                typeof(string),
                typeof(FieldBindingProxy),
                new UIPropertyMetadata(null,
                    (ss, ee) =>
                    {
                        (ss as FieldBindingProxy).InitializeAndSetBinding();
                    }));

        #endregion

        #endregion

        #region Helper Methods

        private void InitializeAndSetBinding()
        {
            if (_owner == null || _grid == null)
                return;

            if (string.IsNullOrEmpty(PropertyPath))
                return;

            Binding binding = new Binding(PropertyPath);

            switch (BindingSource)
            {
                case BindingSource.DataContext:
                    binding.Source = _grid.DataContext;
                    break;
                case BindingSource.XamGrid:
                    binding.Source = _grid;
                    break;
                default:
                    break;
            }

            if (Converter != null)
                binding.Converter = Converter;

            if (ConverterParameter != null)
                binding.ConverterParameter = ConverterParameter;

            FieldInfo fieldInfo = typeof(ColumnBase).GetField(FieldPropertyPath + "Property");
            if (fieldInfo == null)
                fieldInfo = typeof(Column).GetField(FieldPropertyPath + "Property");

            if (fieldInfo == null)
                fieldInfo = _owner.GetType().GetField(FieldPropertyPath + "Property");

            if (fieldInfo == null)
                throw new InvalidOperationException(FieldPropertyPath + " property not found or it is not a DependencyProperty");

            DependencyProperty property = fieldInfo.GetValue(null) as DependencyProperty;

            BindingOperations.SetBinding(_owner, property, binding);
        }

        #endregion

        #region Attached Property

        public static readonly DependencyProperty FieldBindingProxiesProperty =
            DependencyProperty.RegisterAttached(
                "FieldBindingProxies",
                typeof(FieldBindingProxyCollection),
                typeof(FieldBindingProxy),
                new PropertyMetadata(new FieldBindingProxyCollection(),
                    (o, e) =>
                    {
                        FieldVisibilityBindingChanged(o as Column,
                            e.OldValue as FieldBindingProxyCollection,
                            e.NewValue as FieldBindingProxyCollection);
                    }));

        public static FieldBindingProxy GetFieldBindingProxies(DependencyObject target)
        {
            return target.GetValue(FieldBindingProxiesProperty) as FieldBindingProxy;
        }

        public static void SetFieldBindingProxies(DependencyObject target, FieldBindingProxy value)
        {
            target.SetValue(FieldBindingProxiesProperty, value);
        }

        public static void FieldVisibilityBindingChanged(
            Column field,
            FieldBindingProxyCollection oldVal,
            FieldBindingProxyCollection newVal)
        {
            if (field == null)
            {
                return;
            }
            if (oldVal != null)
            {
                foreach (var item in oldVal)
                {
                    item.DetachOwner(field);
                }
            }
            if (newVal != null)
            {
                foreach (var item in newVal)
                {
                    item.AttachOwner(field);
                }
            }
        }

        #endregion

    }

    public enum BindingSource
    {
        /// <summary>
        /// Use to set the source of the Visibility Binding to the DataContext of the XamDataGrid
        /// </summary>
        DataContext,
        /// <summary>
        /// Use to set the source of the Visibility Binding to the XamDataGrid itself, when using RelativeSource 
        /// </summary>
        XamGrid
    }

    public class FieldBindingProxyCollection : ObservableCollection<FieldBindingProxy>
    { }

    #endregion
}
